# frozen_string_literal: true

class Hermes::Api::Dmo::HousingRequestEntityFormLevelsController < Hermes::Api::Dmo::ApplicationController
  before_action :set_entity_administrator, only: %i[update destroy]

  class << self
    def mutate_params
      {
        consent_fields: {type: 'Array'},
        description: {type: 'String'},
        level: {type: 'Integer'},
        delegation: {type: 'Boolean'},
        entity_id: {type: 'String'},
      }
    end 

    def acceptable_params
      {
   
      }.merge(self.mutate_params).merge(super)
    end 

    def valid_operators
      %w[$in $nin $ne $gt $gte $lt $lte $eq $regex $all]
    end

    def regex_params
      {}
    end 
  end
  
  def index
    acceptable_params *self.class.acceptable_params.keys
    valid_operators *self.class.valid_operators

    @entity = if params[:entity_siret]
      Hermes::Entity.find_by(siret: params[:entity_siret])
    elsif params[:entity_id]
      Hermes::Entity.find_by(id: params[:entity_id])
    end
    
    active_hr_entity_id = Hermes::HousingRequest.find_by(is_active: true, user_id: current_user.id)&.entity_id 
    if (current_user_roles & ['cre', 'rdsl', 'entreprise', 'bailleur', 'superadministrateur']).empty? &&
      !(active_hr_entity_id && @entity && active_hr_entity_id == @entity.id)
      return render_json_error(:unauthorized, :cannot_access) 
    end 
 
    return render_json_error(:not_found, :instance_not_found) unless @entity

    form_levels = @entity.form_levels.where(build_query(Hermes::HousingRequestEntityFormLevel)).order(order_params)
    paginate(
      form_levels,
      each_serializer: Hermes::HousingRequestEntityFormLevelSerializer
    )
  end

  def show
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unauthorized, :not_readers) unless @instance.entity&.readable_by?(@user)

    super
  end

  def create
    return render_json_error(:unauthorized, :not_entreprise) unless current_user_roles.include?('entreprise') || current_user_roles.include?('superadministrateur') || current_user_roles.include?('collaborateur entreprise')
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

    super
  end

  private

  def set_instance
    @instance = @model.find_by(id: params[:id]) if params[:id]
  end

  def set_entity_administrator
    return render_json_error(:not_found, :object_not_found) unless @instance
    ur = Hermes::UserRight.find_by(user_id: current_user.id, entity_id: @instance.entity_id, right: 'administrator')
    return render_json_error(:unauthorized, :not_administrator_right_on_entity) unless ur || current_user_roles.include?('superadministrateur') 
  end

  def user_can_destroy?
    # method set_entity_administrator already check current_user_right
    true
  end
end