# frozen_string_literal: true

module Hermes
  class UpdateSubscription
    include Hermes::Concerns::CommonFieldsConcern

    field :name, type: String
    field :object_id, type: BSON::ObjectId
    field :reference_model_name, type: String
    field :email, type: Boolean
    field :web, type: Boolean
    field :slack, type: Boolean
    field :sms, type: Boolean
    field :user_email, type: String
    
    writer_auth_roles 'superadministrateur'

    validates :name, uniqueness: { scope: [:object_id, :reference_model_name] }

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    common_indexes *%i[
      name
      object_id
      reference_model_name
      email
      web
      slack
      sms
    ]
    index(name: 1, object_id: 1, reference_model_name: 1)


    before_save :before_save_actions

    def before_save_actions
      self.user_email = User.find(object_id)&.email  if object_id_changed? && object_id && reference_model_name == 'User'
    end 
  end
end
