# frozen_string_literal: true

module Hermes
  module BoClient
    module Mappings
      module Concerns
        # Abstract class for mappings
        module BaseMap
          extend ActiveSupport::Concern
          
          included do
            def method_missing(method_name)
              @map ||= {}
              super unless @map.keys.include?(method_name)
    
              mapped_element_to_seek = @map[method_name.to_s] || @map[method_name.to_sym]
              object_send(mapped_element_to_seek, main_object)
            end
            
          def respond_to_missing?(method_name, include_private = false)
            @map.include?(method_name) || super
          end
          
          attr_accessor :main_object
          
          # recursively searching for value from a string path
          def object_send(value_path, from)
            method_call_regexp = /^([\w_]+)\.([\w_.]+)$/
            if value_path =~ method_call_regexp
              first_match_group = Regexp.last_match[1]
              second_match_group = Regexp.last_match[2]
              new_from = nil
              if from.respond_to?('@' + first_match_group)
                new_from = from.send('@' + first_match_group)
              elsif from.respond_to?(first_match_group)
                new_from = from.send(first_match_group)
              elsif from.respond_to?('fetch')
                new_from = from.fetch(first_match_group)
              else
                raise ArgumentError, "#{first_match_group} is neither method or key of #{from}"
              end
              if new_from.blank?
                Rails.logger.warn "#{first_match_group} from #{from} is nil"
                return
              end
              
              object_send(second_match_group, new_from)
            else
              begin
                from.send(value_path) ||
                    from.fetch(value_path)
              rescue NoMethodError
                Rails.logger.debug "#{value_path} is neither a method or key of #{from}"
                nil
              end
            end
          end
          end
          
          module ClassMethods
            @map={}
            def base_mapping(**map)
              @map = map
            end
          end
        end
      end
    end
  end
end
