# frozen_string_literal: true

# Return Hermes::HousingRequest attributes from parsed xml

module Hermes
  module BoClient
    module Mappings
      class HousingRequestSupplementMap   
        include Hermes::BoClient::MappingHelper
             
        def initialize(xml)
          @xml = xml
        end

        def convert_to_bool(str)
          return nil if str == nil
          ![false, 0, "0", "f", "F", "false", "FALSE", "off", "OFF"].include?(str)
        end 

        def build_housing_request
          attributes = {
            cotations: {},
            supplement_status: nil,
            supplement_can_apply: nil,
          }
          complementDemande = @xml['complementDemandeReponse']
          localisationsSouhaitees = transform_in_array(complementDemande.dig('localisationsSouhaitees', 'localisationSouhaitee'))

          localisationsSouhaitees&.each do |loc|
            next unless loc
            attributes[:cotations][loc['commune'].to_s] = loc['nbreDePoints'].to_i if loc['nbreDePoints'].present?
          end

          attributes[:supplement_status] = complementDemande.dig('etatDemande', 'libelleEtatDemande')
          attributes[:supplement_can_apply] = convert_to_bool(complementDemande.dig('etatDemande', 'candidatAutorise'))

          attributes
        end
      end
    end
  end
end
