# frozen_string_literal: true

# exemple d'utilisation
=begin
ha = Hermes::HousingApplication.first
so=Hermes::BoClient::ApplicationTask.new(
    {
                                                 housing_request_id: ha.housing_request_id,
                                                 housing_offer_id: ha.housing_offer_id,
                                             }
)
so.to_xml
ExecuteSopraTaskJob.perform_now(so.endpoint, so.to_xml)
=end

module Hermes
  module BoClient
    class ApplicationTask < BaseTask
      
      def initialize(options = {})
        @xsd = "StatutProposition.xsd"
        super(options)
      end
      
      endpoint {'/ws/rest/als/demande/v1/indiquerCandidature'}
      
      task_name 'application'
      type_flux 'BIEN'
      mapping 'Hermes::BoClient::Mappings::HousingApplicationMap'
      
      params *%i[
      refInterneDemande
      refInterneOffre
      ]
      
      def to_xml
        namespaces = {
            'xmlns:ns2' => 'http://v1.als.ikos.webservice.immobilier.sopra/demande/candidature'
        }
        builder = Nokogiri::XML::Builder.new(encoding: 'UTF-8') do |b|
              b['ns2'].candidature(namespaces) do
             
                 b.refInterneDemande mapping.refInterneDemande do
                   b.parent.namespace = nil
                 end
                 b.refInterneOffre mapping.refInterneOffre do
                   b.parent.namespace = nil
                 end
                 b.datePositionnement Date.today do
                   b.parent.namespace = nil
                 end
             
          end
        end
        builder.to_xml
      end
    end
  end
end
