
require 'csv'

def run_script
  application_query = {
    status: 'TRANSMIS BAILLEUR',
    :status_updated_at.gte => (DateTime.current - 14.days)
  }

  validated_applications = Hermes::HousingApplication.where(
    application_query
  )

  validated_applications_count = validated_applications.count

  CSV.open('script_devops_8668_reprise_public_share.csv', "ab") do |csv|
    csv << ['housing_application_id', 'public_share_id', 'status']

    validated_applications.batch_size(5).each_with_index do |validated_application, idx|
      puts "#{idx+1}/#{validated_applications_count} - Start public share refresh token for application #{validated_application.id} "
      row = []
      row.push(validated_application.id)
      begin
        housing_request   = validated_application.housing_request
        housing_offer     = validated_application.housing_offer
        if housing_offer.lessor
          public_share = Hermes::HousingRequestShare.find_or_create_by(lessor_id: housing_offer.lessor.id, housing_request_id: housing_request.id)
          public_share.update(code: public_share.generate_code, expire_at: public_share.generate_expiration)
          row.push(public_share.id)
          row.push("updated : #{public_share.public_url}")
          csv << row
          puts "PublicShare #{public_share.id} updated !"
        else
          puts "No lessor found for offer #{housing_offer.id} can't create or update public share"
          row.insert(3, "error : No lessor found for offer #{housing_offer.id} can't create or update public share")
          csv << row
        end
      rescue => exception
        row.insert(3, "error : #{exception.inspect}")
        csv << row
      end
    end
  end
end

run_script