# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'

dir_path = File.dirname(__FILE__)
schema_path = [dir_path, 'demandeOrientationsReponse.xsd'].join('/')
require [Hermes::Engine.root, %w[lib hermes bo_client tasks]].flatten.join('/').to_s

RSpec.describe Hermes::BoClient::UpdateOrientationTask, type: :helper do
  let!(:xsd) { Nokogiri::XML::Schema(File.read(schema_path)) }
  let!(:hr) { create(:active_request, als_id: '0000001')}

  it 'should load a valid XSD' do
    expect(xsd).to be_present
    expect(xsd.class).to be Nokogiri::XML::Schema
  end

  it 'should has all values' do
    so = Hermes::BoClient::UpdateOrientationTask.new(housing_request_id: hr.id)

    expect(so.endpoint).to eq('/ws/rest/als/demande/v1/demandeOrientations/0000001')
    expect(so.to_xml).to eq(nil)
  end

  it 'update housing_request orientations' do
    so = Hermes::BoClient::UpdateOrientationTask.new(housing_request_id: hr.id)

    so.execute
    hr.reload
    expect(hr.housing_channel_keys).to include('priority_public')
    expect(hr.housing_channel_keys).to include('dalo')
    expect(hr.updated_orientation_at).not_to be_nil
  end

  it 'when error' do
    hr.update(als_id: '0000002')
    so = Hermes::BoClient::UpdateOrientationTask.new(housing_request_id: hr.id)

    so.execute
    hr.reload
    expect(hr.housing_channel_keys).not_to include('priority_public')
    expect(hr.housing_channel_keys).not_to include('dalo')
    expect(hr.updated_orientation_at).to be_nil

    Hermes::LogJob.perform_now
    expect(Log.where(level: 'error',  code: "UPDATE_ORIENTATION_ERROR").count).to eq(1)
  end

  it 'when no keys' do
    hr.update(als_id: '0000003')
    so = Hermes::BoClient::UpdateOrientationTask.new(housing_request_id: hr.id)

    so.execute
    hr.reload
    expect(hr.housing_channel_keys).to eq(['dc'])
    expect(hr.updated_orientation_at).not_to be_nil
  end


  it 'when 1 key' do
    hr.update(als_id: '0000004')
    so = Hermes::BoClient::UpdateOrientationTask.new(housing_request_id: hr.id)

    so.execute
    hr.reload
    Hermes::LogJob.perform_now

    expect(Log.where(level: 'error',  code: "UPDATE_ORIENTATION_ERROR").count).to eq(0)
    expect(hr.housing_channel_keys).to include('priority_public')
    expect(hr.updated_orientation_at).not_to be_nil
  end
end
