require 'rails_helper'
require [Hermes::Engine.root, %w[app lib notifications_center user]].flatten.join('/').to_s

def perform_job_now(job)
  job[:job].perform_now(*deserialize_argument(job[:args]))
end 

RSpec.describe NotificationsCenter::User, type: :helper do
  let!(:user) { create(:user, email: 'test1@test.com') }
  let!(:entity) {create(:entity, siret: '12345678901234', title: 'Compagny')}
  let!(:applicant) { create(:applicant, user: user, main_employer_siret: entity.siret) }
  let!(:hr) { create(:housing_request, is_active: true, main_user_id: user.id, main_applicant: applicant) }
  let!(:offer1) {create(:housing_offer, insee_code: '75000', surface: 56, typology: 'T2', rent_with_charges: 840)}
  let!(:offer2) {create(:housing_offer, insee_code: '75000', surface: 32, typology: 'T1', rent_with_charges: 500)}

  before do 
    # create default template
    MailTemplate.create(default: true, body: '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
      <html style="margin:0 auto !important;padding:0 !important;height:100% !important;width:100% !important;">
        <head>
          <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
          <meta name="viewport" content="width=device-width">
          <meta http-equiv="X-UA-Compatible" content="IE=edge">
          <meta name="x-apple-disable-message-reformatting">
          <title>{{title}}</title>
          <!-- CSS Reset : BEGIN -->
          <!-- CSS Reset : END -->
          <!-- Progressive Enhancements : BEGIN -->
          <!-- Progressive Enhancements : END -->
          <style>
            a:link{color:#FFFFFF;text-decoration:none}
            a:visited{text-decoration:none;color:#FFFFFF}
            a:hover{text-decoration:none;color:#FFFFFF}
            a:active{text-decoration:none;color:#FFFFFF}
          </style>
          <style type="text/css">
                        *[x-apple-data-detectors],  /* iOS */
                        .x-gmail-data-detectors,    /* Gmail */
                        .x-gmail-data-detectors *,
                        .aBn {
                            border-bottom: 0 !important;
                            cursor: default !important;
                            color: inherit !important;
                            text-decoration: none !important;
                            font-size: inherit !important;
                            font-family: inherit !important;
                            font-weight: inherit !important;
                            line-height: inherit !important;
                        }

                        @media only screen and (min-device-width: 375px) and (max-device-width: 413px) { /* iPhone 6 and 6+ */
                            .email-container {
                                min-width: 100% !important;
                            }
                        }
                        body{
                          font-family: Arial; 
                          font-size: 14px; 
                          color: #333333; 
                          line-height: 1.4;
                        }
      
                        table td {
                              border-collapse: collapse;
                          
                          }
      
                          table th {
                              border-collapse: collapse;
                          }
      
                          table {
                              border-collapse: collapse;
                          }
      
      
                    /* Media Queries */
                    @media screen and (max-width: 600px) {
                        .email-container {
                            width: 100% !important;
                            margin: auto !important;
                        }
                        .fluid {
                            width: 100% !important;
                            max-width: 100% !important;
                            height: auto !important;
                            margin-left: auto !important;
                            margin-right: auto !important;
                        }
      
                        .fluid.logo {
                            max-width: 200px !important;
                        }
      
                        /* What it does: Forces table cells into full-width rows. */
                        .stack-column,
                        .stack-column-center {
                            display: block !important;
                            width: 100% !important;
                            max-width: 100% !important;
                            direction: ltr !important;
                            padding: 0 !important;
                        }
                        /* And center justify these ones. */
                        .stack-column-center {
                            text-align: center !important;
                        }
                        /* What it does: Generic utility class for centering. Useful for images, buttons, and nested tables. */
                        .center-on-narrow {
                            text-align: center !important;
                            display: block !important;
                            margin-left: auto !important;
                            margin-right: auto !important;
                            float: none !important;
                        }
                        table.center-on-narrow {
                            display: inline-block !important;
                        }
                        .header-table{
                            font-size: 16px;
                        }
                        
                        .show-on-mobile{
                          display: inline!important;
                        }
      
                        .hide-on-mobile{
                            display: none!important;
                        }
      
                        .mobile-no-padding{
                            padding: 0 !important;
                        }
                        .mobile-margin-auto{
                            margin-left: auto !important;
                            margin-right: auto !important;
                        }
                        
                        .stack-column > table{
                          min-height: 0!important;
                          height: auto!important;
                        }
      
                        .stack-column.ol-item{
                            padding-top: 15px !important;
                        }
      
                        .stack-column.ol-body{
                          padding: 10px 0 !important;
                          width: 90% !important;
                          max-width: 90% !important;
                          margin: auto;
                        }
                    }
          </style>
        </head>
        <body style="margin:0 auto !important;padding:0 6px !important;height:100% !important; margin:0; background-color:#f7f7f7;">
          <table align="center" border="0" cellpadding="0" cellspacing="0" width="600" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; margin: auto; max-width: 600px!important; width: 600px; background-color: white;" class="email-container" role="presentation">
            <tbody>
              <tr>
                <td style="text-align:center;font-family: Arial;font-size: 14px;color: #333333;padding: 8px 0;line-height: 1.4;font-style: italic;">{{title}}</td>
              </tr>
              <tr>  
              <td>
                <!-- Logo -->
                <table role="presentation" cellspacing="0" cellpadding="0" border="0" align="center" width="100%" style="border-spacing:0 !important;border-collapse:collapse !important;table-layout:fixed !important;max-width: 600px;" class="email-container">
                  <tr>
                    <td style="padding: 8px;">
                        <a href="https://www.actionlogement.fr" target="_blank">
                          <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/f953d58026b3288f6bac326f3fdafb21/6141affbb93ee3484e3a0795/logo_al.png" height="40" class="fluid logo" alt="Action Logement">
                        </a>
                    </td>
                    <td style="padding: 8px;">
                      <a href="https://als-hermes-scoring-rec.be-idys.com" target="_blank" style="float: right" >
                        <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/9adf36a551889157a2bf6340e8238523/6141aedeb93ee3484e3a0776/logo_alin_copy.png"  height="40" class="fluid logo" alt="Action Logement" alt="Alin">
                      </a>
                    </td>
      
                  </tr>
                </table>
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="background-color: #abe8e6; color: #ffffff; font-size: 26px; font-family: Arial; font-weight: bold; text-transform: uppercase; text-align: center; line-height: 1.5; border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; max-width: 600px;" class="email-container header-table" role="presentation">
                <tbody>
                <tr>
                  <td>
                    <img border="0" height="190" width="600" style="-ms-interpolation-mode: bicubic; display: block; white-space: pre;" class="fluid" src="https://als-hermes-scoring-rec.be-idys.com/sassets/48693a1e251e5fb06ff8389f97ce9736/6141b0007a6d0043058d780c/asset-1631694848-h0o8c2rh.JPG" alt="Alin Salaries">
                  </td>
                </tr>
                </tbody>
                </table>
              </td>
            </tr>
            
            </tbody>
          </table>
          <table align="center" border="0" cellpadding="0" cellspacing="0" width="600" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; margin: auto; max-width: 600px!important; width: 600px; background-color: white;" class="email-container" role="presentation">
            <tbody>
            <tr>
              <td>
                <!-- Contenu liste numérotée -->
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="border-spacing: 0 !important; border-collapse: collapse !important; table-layout: fixed !important; max-width: 600px; margin: 0 15px; width: auto;  background-color: white;" class="email-container" role="presentation">
                <tbody>
                  {{content}} 
                </tbody>
                </table>
                  <!-- Footer AL -->
                <table align="center" border="0" cellpadding="0" cellspacing="0" width="100%" style="border-spacing: 0 !important; border-collapse: separate !important; table-layout: fixed !important; max-width: 600px; padding-top: 10px; background-color: white;" class="footer-rs email-container" role="presentation">
                  <tbody>
                  <tr>
                    <td align="center" style="padding-top: 64px">
                      <img src="https://als-hermes-scoring-rec.be-idys.com/sassets/92ee6d1d29d4446c7320b5fdccee21de/617a9023b93ee34ab41a3106/separation_transparent.png" alt="separation">
                    </td>
                  </tr>
                  <tr>
                    <td style="font-weight: 800;color: #003f7a; text-align: center; padding: 32px">
                      <span style="font-size: large;">AYEZ LE REFLEXE ACTION LOGEMENT !</span><br/>
                      <a style="color: #003f7a !important;" href="https://www.actionlogement.fr" target="_blank">www.actionlogement.fr</a>
                    </td>
                  </tr>
                  <tr>
                      <td style="font-size: 12px; text-align: center; padding: 32px; color: white; background: #e41d4b;">
                      <p>
                        <a style="color:rgb(0, 0, 238)" href="https://als-hermes-entreprise-rec.be-idys.com/#/modif-account?mail=true" target="_blank"> <u>Annuler l abonnement à nos emails</u> </a><br />
                        Les mails envoyés à cette adresse ne sont pas traités, si vous avez une autre question merci de vous rendre sur votre espace en ligne.
                      </p>
                      <p>&nbsp;</p>
                      <p>
                        Action Logement Services - SAS au capital de 20.000.000 d euros <br>
                        Siège social : 19/21 quai d Austerlitz 75013 Paris - 824 541 148 RCS Paris <br>
                        Immatriculée à l ORIAS sous le numéro 17006232 - Société de financement agréée et contrôlée par l ACPR
                      </p>
                    </td>
                  </tbody>
                </table>
              </td>
            </tr>
            </tbody>
          </table>
        </body>
      </html>')
  end 

  # TODO : Refacto to not duplicate code
  it 'new_reserved_offer' do
    expect(hr.entity_id).to eq(entity.id)

    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:new_reserved_offer],
      channels: ['email'],
      datas: {offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.new_reserved_offer')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.new_reserved_offer')
    
    # send notification
    notif.send_notification
    last_email_job = enqueued_jobs.select{|job| job[:job] == ActionMailer::DeliveryJob }.last
    perform_job_now(last_email_job)

    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement Services AL'In Logement exclusif Compagny")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end

  it 'new_published_offer' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:new_published_offer],
      channels: ['email'],
      datas: {offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.new_published_offer')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.new_published_offer')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL'in_De nouvelles offres sont disponibles")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
  it 'application_refusal' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:application_refusal],
      channels: ['email'],
      datas: {   
        first_name: "martin",
        last_name: "matin" , 
        als_id: "33",
        city: "Ales",
      offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.application_refusal')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.application_refusal')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL’in_Information sur votre candidature")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
  it 'application_offer_taken_back' do 
    # Launch notification
    user.notify(
      key: NOTIFICATION_KEYS[:user][:application_offer_taken_back],
      channels: ['email'],
      datas: {   
        first_name: "martin",
        last_name: "matin" , 
        als_id: "33",
        city: "Ales",
       applicated_at:  DateTime.new(2007,11,19,8,37,48,"-06:00").strftime("%d/%m/%Y"),
      offer_ids: [offer1.id.to_s, offer2.id.to_s]}
    )

    # select notification job 
    last_notif_job = enqueued_jobs.select{|job| job[:job] == AsyncNotifierJob }.last
    expect(last_notif_job[:args][0]['key']).to eq('user.application_offer_taken_back')
    
    # perform notification job and check on notif model created
    perform_job_now(last_notif_job)
    notif = ::NotificationCenter::Notification.last
    expect(notif.key).to eq('user.application_offer_taken_back')
    
    # test content notification
    contents = ::NotificationsCenter::User.new(instance: notif)
    expect(contents.title).to eq("Action Logement AL’in_Information sur votre candidature")
    expect(contents.body).to include(offer1.id.to_s, offer2.id.to_s)
  end 
end
